# # 125: Terminal flux calculation via test functions
# ([source code](SOURCE_URL))

module Example125_TestFunctions1D 
using Printf
using VoronoiFVM



function main(;n=100,Plotter=nothing,verbose=false,dense=false)
    h=1/n
    grid=VoronoiFVM.Grid(collect(0:h:1))
    
        
    eps=[1,1.0e-1]

    physics=VoronoiFVM.Physics(
         num_species=2,
    reaction=function(f,u,node,data)
        f[1]=10*(u[1]-u[2])
        f[2]=10*(u[2]-u[1])
    end,

    flux=function(f,u,edge,data)   
        uk=viewK(edge,u)
        ul=viewL(edge,u)
        f[1]=eps[1]*(uk[1]-ul[1])
        f[2]=eps[2]*(uk[2]-ul[2])
    end,
    
    
    storage=function(f,u,node,data)
        f[1]=u[1]
        f[2]=u[2]
    end
    )

    if dense
        sys=VoronoiFVM.DenseSystem(grid,physics)
    else
        sys=VoronoiFVM.SparseSystem(grid,physics)
    end
    
    enable_species!(sys,1,[1])
    enable_species!(sys,2,[1])

    boundary_neumann!(sys,1,1,0.01)
    boundary_dirichlet!(sys,2,2,0.0)

    factory=VoronoiFVM.TestFunctionFactory(sys)
    tf1=testfunction(factory,[2],[1])
    tf2=testfunction(factory,[1],[2])
    
    
    U=unknowns(sys)
    inival=unknowns(sys)
    inival[2,:].=0.1
    inival[1,:].=0.1
    
    control=VoronoiFVM.NewtonControl()
    control.verbose=verbose
    control.damp_initial=0.1
    I1=0
    for xeps in [1.0,0.1,0.01]
        eps=[xeps,xeps]
        solve!(U,inival,sys,control=control)
        I1=integrate(sys,tf1,U)
        
        inival.=U
        if isplots(Plotter)
            p=Plotter.plot()
            Plotter.plot!(p,grid.coord[1,:],U[1,:])
            Plotter.plot!(p,grid.coord[1,:],U[2,:])
            Plotter.gui(p)
        end
        u5=U[5]
    end
    return I1[1]
end

function test()
    main(dense=false) ≈ 0.01 &&
        main(dense=true) ≈ 0.01
end
end
